#ifndef __ZORPLIBCONFIG_H__
#define __ZORPLIBCONFIG_H__

#define ZORPLIB_ENABLE_DEBUG 0
#define ZORPLIB_ENABLE_TRACE 0
#define ZORPLIB_ENABLE_MEM_TRACE 0
#define ZORPLIB_ENABLE_STACKDUMP 0

#define ZORPLIB_ENABLE_CAPS 1
#define ZORPLIB_ENABLE_SSL_ENGINE 1
#define ZORPLIB_ENABLE_TOS 1

#endif /* __ZORPLIBCONFIG_H__ */
